' ---------------------------------------------------------------------------
' AllianceACH XML Requester Service
' 
' VBScript sample illustrating how to interact with AACH services using
' XMLDOMDocument and XMLHTTP.
'
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
' ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
' TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
' PARTICULAR PURPOSE.
'
' Use of this software constitutes acceptance of the end-user license 
' agreement found in the file EULA.doc, which you should have received
' with this software.
'
' Copyright (C) 2003.  Alliance Payment Technologies, Inc. 
' All rights reserved.
'
' ---------------------------------------------------------------------------
option explicit

'
' The URL at which AACH receives XML requests
'
const strAACHRequesterURL = "https://gateway.allianceach.com/requester/request-v2-test.asp"

'
' The ProgID for MSXML DOM Document.  Change this to an older ProgID to use
'   a version of MSXML older than version 4
'
const strMSXMLProgID = "Msxml2.DOMDocument.4.0"

'
' The ProgID for XMLHTTP.  Change this to an older ProgID to use a 
' version of XMLHTTP older than version 4
'
const strXMLHTTPProgID = "Msxml2.XMLHTTP.4.0"

' 
' Class AACH represents a "session" - the context in which a sequence of service
' requests are made.
'
class AACH
  private strUser
  private strPassword
  private objLastResponse
  
  '
  ' Class default constructor (the only one VBScript allows)
  '
  private sub Class_Initialize
    set objLastResponse = CreateObject(strMSXMLProgID)
  end sub

  '
  ' Class destructor
  '
  private sub Class_Terminate
    set objLastResponse = Nothing
  end sub
  
  '
  ' Set the user ID to be used in service requests
  ' 
  property let user(aUser)
    strUser = aUser
  end property
  
  '
  ' Set the password to the used in service requests
  '
  property let password(aPassword)
    strPassword = aPassword
  end property
  
  '
  ' Create a AACHRequest wrapper, with authentication
  '
  ' Returns:
  '   XMLDOMDocument holding the AACHRequest wrapper
  '
  public function CreateRequest()
    '
    ' Create an empty XMLDOMDocument
    '
    dim doc
    set doc = CreateObject(strMSXMLProgID)

    '
    ' Set some DOM properties for best performance in this application
    '
    doc.async = false
    doc.setProperty "SelectionLanguage","XPath"   ' we use XPath
    doc.setProperty "NewParser", true         ' Use the new parser, it's faster

    '
    ' Create a new <AACHRequest> element and add it to the DOM (it's the document element).
    '
    dim root
    set root = doc.appendChild(doc.createElement("AACHRequest"))

    '
    ' Create an <Authentication> element and append it to the document element
    '
    dim auth
    set auth = root.appendChild(doc.createElement("Authentication"))

    '
    ' Create <Username> and <Password> elements and append them to the <Authentication>
    '
    auth.appendChild(doc.createElement("Username")).text = strUser
    auth.appendChild(doc.createElement("Password")).text = strPassword

    '
    ' Return the new document
    '
    set CreateRequest = doc
  end function

  '
  ' Add an empty <Request> element to an existing request
  '
  ' Parameters:
  '   doc       - document in which new XML DOM elements will be created
  '   strID     - Service identifier: becomes the 'ID' attribute of the <Request> element
  '   strType   - Service type (ABASearch, SearchTransactions, ATMVerify, etc...)
  '
  ' Returns:
  '   an XMLDOMElement object which holds the new <Request> element 
  '
  public function CreateRequestElement ( doc, strID, strType )
    '
    ' Create a <Request> element and append it to the document element
    '
    dim aachrequest
    set aachrequest = doc.documentElement.appendChild(doc.createElement("Request"))
    aachrequest.setAttribute "ID",strID

    '
    ' Create <RequestType> element and append it to the <Request> element
    '
    aachrequest.appendChild(doc.createElement("RequestType")).text = strType
    aachrequest.appendChild(doc.createElement("ABANumber")).text = "322282603"

    '
    ' Return the <Request> element
    '
    set CreateRequestElement = aachrequest
  end function  


  '
  ' Send a service request & parse the response.
  '
  ' Returns:  Nothing
  '
  ' An XMLDOMDocument containing the service response is saved in
  ' this.m_lastResponse
  '
  public sub SendRequest (doc)
    '
    ' Create an XMLHTTP instance 
    '
    dim req
    set req = CreateObject(strXMLHTTPProgID)

    '
    ' Send the request & get the response
    '
    req.open "POST",strAACHRequesterURL, false
    req.send doc

    '
    ' Check for HTTP errors
    '
    if req.Status >= 300 then
      Err.Raise 507,,"HTTP request failed: " & req.StatusText
      exit sub
    end if

    ' 
    ' Load the response into an XMLDOMDocument and check for parse errors
    '
    if not objLastResponse.load(req.responseStream) then
      Err.Raise 507,,"Invalid XML response received: " & objLastResponse.parseError.reason & " at line " & objLastResponse.parseError.line & " column " & objLastResponse.parseError.linePos
      exit sub
    end if

    '
    ' Check for successful service invocation (this only tests that the entire
    ' request was received and processed - individual <Request> elements may have
    ' had errors)
    '
    dim wrapperStatus 
    wrapperStatus = GetResponseValue("/AACHRequest/ResponseSummary/Error")
    if wrapperStatus = "True" then
      Err.Raise 507,,"Service authentication failed: " & GetResponseValue("/AACHRequest/ResponseSummary/Message")
      exit sub
    end if
  end sub

  '
  ' Get a single element from the most recent response
  '
  ' Parameters:
  '   strPath - XPath expression specifying the node to be returned
  '
  ' Returns:
  '   XMLDOMNode - The specified node, if it exists, or null otherwise.
  '
  public function GetResponseElement( strPath )
    set GetResponseElement = objLastResponse.selectSingleNode( strPath )
  end function

  '
  ' Get several elements from the most recent response
  '
  ' Parameters:
  '   strPath - XPath expression specifying the node(s) to be returned
  '
  ' Returns:
  '   XMLDOMNodeList - The specified node(s), if they exists, or null otherwise.
  '
  public function GetResponseElementList ( strPath )
    set GetResponseElementList = objLastResponse.selectNodes( strPath )
  end function

  '
  ' Get a single element value from the most recent response
  '
  ' Parameters:
  '   strPath - XPath expression specifying the node to be returned
  '
  ' Returns:
  '   The textual content of the specified node, if it exists, or null otherwise.
  '
  public function GetResponseValue ( strPath )
    dim node
    set node = GetResponseElement( strPath )
    if node is Nothing then
      GetResponseValue = ""
    else
      GetResponseValue = node.text
    end if
  end function
end class

'
' Sample "program" illustrating how to use the AACH object
' (This program is intended to be run using "cscript" - the command-line version of
' Windows Scripting Host).
'
dim stdout
set stdout = WScript.StdOut

' Check for required command line arguments
dim args
set args = WScript.Arguments
if 2 <> args.length then
  Err.Raise 5,,"Usage: CallAACH username password"
end if

' Create a AACH session object & set the credentials
dim session
set session = New AACH
session.user = args(0)
session.password = args(1)

' Create a ABASearch service request
dim doc
set doc = session.CreateRequest ()
dim service
set service = session.CreateRequestElement(doc,"0","ABASearch")

' Send the service request
session.SendRequest(doc)

'
' Check for successful service
'
dim statusCode 
statusCode = session.GetResponseValue( "/AACHRequest/Request/Status")
if statusCode <> "Success" then
  Err.Raise 507,,"Service request failed: " & session.GetResponseValue("/AACHRequest/Request/StatusMessage")
end if

'
' Get the resultsItems & print them out
'
dim results 
set results = session.GetResponseElementList("/AACHRequest/Request/Results/Result")
if results.length <> 0 then
  dim i
    for i = 0 to results.length-1 
    stdout.WriteLine "Result " & i
    dim result
    set result = results.item(i)
    dim node
    set node = result.selectSingleNode("ABANumber")
    if not (node is Nothing) then
      stdout.WriteLine "    ABA Number: " & node.text
    end if
    set node = result.selectSingleNode("Name")
    if not (node is Nothing) then
      stdout.WriteLine "     Bank Name: " & node.text
    end if
    set node = result.selectSingleNode("Phone")
    if not (node is Nothing) then
      stdout.WriteLine "  Phone Number: " + node.text
    end if
    stdout.WriteLine ""
  next
else
  stdout.WriteLine "No results were found"
end if

' End of CallAACH.vbs
